package com.persagy.account.pojo.dto;

import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 账号-角色关联信息
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Getter
@Setter
@ToString
@EqualsAndHashCode(callSuper = false)
@TableName("saas_account_role")
public class SaasAccountRole extends Model<SaasAccountRole> {
	private static final long serialVersionUID = -6223601411225948734L;

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String groupCode; // 集团编码

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String accountId; // 账号ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String roleId; // 角色ID
	
	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String casType;	//关联类型，0-主岗，1-副岗，2-业务超管默认角色
	
	@TableField(exist = false)
	private List<SaasRole> roles; // 角色信息集合
	
	@TableField(exist = false)
	private List<SaasMenu> menus; // 菜单信息集合-树形结构

	public static class Builder {

		private LambdaQueryWrapper<SaasAccountRole> queryWrapper = null;

		private LambdaUpdateWrapper<SaasAccountRole> updateWrapper = null;

		public Builder createQueryWrapper() {
			queryWrapper = new LambdaQueryWrapper<SaasAccountRole>();
			return this;
		}

		public Builder createUpdateWrapper() {
			updateWrapper = new LambdaUpdateWrapper<SaasAccountRole>();
			return this;
		}

		public Builder groupCodeEq(String groupCode) {
			if (StringUtil.isNotBlank(groupCode)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountRole::getGroupCode, groupCode);
				} else {
					queryWrapper.eq(SaasAccountRole::getGroupCode, groupCode);
				}
			}
			return this;
		}

		public Builder accountIdEq(String accountId) {
			if (StringUtil.isNotBlank(accountId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountRole::getAccountId, accountId);
				} else {
					queryWrapper.eq(SaasAccountRole::getAccountId, accountId);
				}
			}
			return this;
		}
		
		public Builder accountIdIn(Set<String> accountIds) {
			if (CollectionUtil.isNotEmpty(accountIds)) {
				if (updateWrapper != null) {
					updateWrapper.in(SaasAccountRole::getAccountId, accountIds);
				} else {
					queryWrapper.in(SaasAccountRole::getAccountId, accountIds);
				}
			}
			return this;
		}
		
		public Builder roleIdEq(String roleId) {
			if (StringUtil.isNotBlank(roleId)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountRole::getRoleId, roleId);
				} else {
					queryWrapper.eq(SaasAccountRole::getRoleId, roleId);
				}
			}
			return this;
		}
		
		public Builder roleIdIn(Set<String> roleIds) {
			if (CollectionUtil.isNotEmpty(roleIds)) {
				if (updateWrapper != null) {
					updateWrapper.in(SaasAccountRole::getRoleId, roleIds);
				} else {
					queryWrapper.in(SaasAccountRole::getRoleId, roleIds);
				}
			}
			return this;
		}
		
		public Builder casTypeEq(String casType) {
			if (StringUtil.isNotBlank(casType)) {
				if (updateWrapper != null) {
					updateWrapper.eq(SaasAccountRole::getCasType, casType);
				} else {
					queryWrapper.eq(SaasAccountRole::getCasType, casType);
				}
			}
			return this;
		}
		
		public LambdaQueryWrapper<SaasAccountRole> builderQueryWrapper() {
			return queryWrapper;
		}

		public LambdaUpdateWrapper<SaasAccountRole> builderUpdateWrapper() {
			return updateWrapper;
		}
	}
}
