package com.persagy.account.pojo.dto;

import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.google.common.collect.Sets;
import com.persagy.common.annotation.IdGeneratorPrefix;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.model.BaseEntity;
import com.persagy.common.utils.StringUtil;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * 产品页面配置
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-05-06 10:12:47
 */
@Getter
@Setter
@ToString
@EqualsAndHashCode(callSuper = false)
@TableName("saas_product_page")
@IdGeneratorPrefix(SaasCommonConstant.SAAS_PRODUCT_PAGE_ID_PREFIX)
public class SaasProductPage extends BaseEntity<SaasProductPage> {
	private static final long serialVersionUID = 448704137255167187L;

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String productId; // 产品ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String parentId; // 上级页面ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String pageName; // 页面名称

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String pageIcon; // 页面图标

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String pageUrl; // 页面URL，一般为前端页面跳转链接

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String pageAngle; // 页面视角，0-集团项目页，1-集团独有页，2-项目独有页，默认0

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String pageFloor; // 是否为落地页，0-非落地页，1-集团落地页，2-项目落地页，3-集团项目落地页

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private Integer pageLevel;	// 页面层级
	
	@TableField(fill = FieldFill.INSERT, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date createTime; // 创建时间

	@TableField(fill = FieldFill.INSERT_UPDATE, updateStrategy = FieldStrategy.NOT_EMPTY)
	private Date updateTime; // 更新时间

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String updateUser; // 最后一次操作者ID

	@TableField(updateStrategy = FieldStrategy.NOT_EMPTY)
	private String remark; // 备注
	
	@TableField(exist = false)
	private List<SaasProductPage> childrens; 	// 下级所有页面集合
	
	@TableField(exist = false)
	private List<SaasFunction> functions; 		// 本页面对应的权限集合

	public static class BuilderQueryWrapper {

		private LambdaQueryWrapper<SaasProductPage> queryWrapper = new LambdaQueryWrapper<SaasProductPage>();

		public BuilderQueryWrapper idEq(String id) {
			if (StringUtil.isNotBlank(id)) {
				queryWrapper.eq(SaasProductPage::getId, id);
			}
			return this;
		}

		public BuilderQueryWrapper productIdEq(String productId) {
			if (StringUtil.isNotBlank(productId)) {
				queryWrapper.eq(SaasProductPage::getProductId, productId);
			}
			return this;
		}

		public BuilderQueryWrapper parentIdEq(String parentId) {
			if (StringUtil.isNotBlank(parentId)) {
				queryWrapper.eq(SaasProductPage::getParentId, parentId);
			}
			return this;
		}

		public BuilderQueryWrapper pageNameEq(String pageName) {
			if (StringUtil.isNotBlank(pageName)) {
				queryWrapper.eq(SaasProductPage::getPageName, pageName);
			}
			return this;
		}

		public BuilderQueryWrapper pageIconEq(String pageIcon) {
			if (StringUtil.isNotBlank(pageIcon)) {
				queryWrapper.eq(SaasProductPage::getPageIcon, pageIcon);
			}
			return this;
		}

		public BuilderQueryWrapper pageUrlEq(String pageUrl) {
			if (StringUtil.isNotBlank(pageUrl)) {
				queryWrapper.eq(SaasProductPage::getPageUrl, pageUrl);
			}
			return this;
		}

		public BuilderQueryWrapper pageAngleEq(String pageAngle) {
			if (StringUtil.isNotBlank(pageAngle)) {
				queryWrapper.eq(SaasProductPage::getPageAngle, pageAngle);
			}
			return this;
		}
		
		public BuilderQueryWrapper pageAngleIn(String angle) {
			if (StringUtil.isNotBlank(angle)) {
				if (SaasCommonConstant.STR_STATUS_1.equals(angle)) {
					queryWrapper.in(SaasProductPage::getPageAngle, Sets.newHashSet(SaasCommonConstant.STR_STATUS_0, SaasCommonConstant.STR_STATUS_1));
				} else {
					queryWrapper.in(SaasProductPage::getPageAngle, Sets.newHashSet(SaasCommonConstant.STR_STATUS_0, SaasCommonConstant.STR_STATUS_2));
				}
			}
			return this;
		}

		public BuilderQueryWrapper pageFloorEq(String pageFloor) {
			if (StringUtil.isNotBlank(pageFloor)) {
				queryWrapper.eq(SaasProductPage::getPageFloor, pageFloor);
			}
			return this;
		}
		
		public BuilderQueryWrapper pageLevelEq(String pageLevel) {
			if (StringUtil.isNotBlank(pageLevel)) {
				queryWrapper.eq(SaasProductPage::getPageLevel, pageLevel);
			}
			return this;
		}

		public BuilderQueryWrapper createTimeEq(Date createTime) {
			if (null != createTime) {
				queryWrapper.eq(SaasProductPage::getCreateTime, createTime);
			}
			return this;
		}

		public BuilderQueryWrapper updateTimeEq(Date updateTime) {
			if (null != updateTime) {
				queryWrapper.eq(SaasProductPage::getUpdateTime, updateTime);
			}
			return this;
		}

		public BuilderQueryWrapper updateUserEq(String updateUser) {
			if (StringUtil.isNotBlank(updateUser)) {
				queryWrapper.eq(SaasProductPage::getUpdateUser, updateUser);
			}
			return this;
		}

		public BuilderQueryWrapper remarkEq(String remark) {
			if (StringUtil.isNotBlank(remark)) {
				queryWrapper.eq(SaasProductPage::getRemark, remark);
			}
			return this;
		}

		public LambdaQueryWrapper<SaasProductPage> builder() {
			return queryWrapper;
		}

	}

	public static class BuilderUpdateWrapper {

		private LambdaUpdateWrapper<SaasProductPage> updateWrapper = new LambdaUpdateWrapper<SaasProductPage>();

		public BuilderUpdateWrapper idEq(String id) {
			if (StringUtil.isNotBlank(id)) {
				updateWrapper.eq(SaasProductPage::getId, id);
			}
			return this;
		}

		public BuilderUpdateWrapper productIdEq(String productId) {
			if (StringUtil.isNotBlank(productId)) {
				updateWrapper.eq(SaasProductPage::getProductId, productId);
			}
			return this;
		}

		public BuilderUpdateWrapper parentIdEq(String parentId) {
			if (StringUtil.isNotBlank(parentId)) {
				updateWrapper.eq(SaasProductPage::getParentId, parentId);
			}
			return this;
		}

		public BuilderUpdateWrapper pageNameEq(String pageName) {
			if (StringUtil.isNotBlank(pageName)) {
				updateWrapper.eq(SaasProductPage::getPageName, pageName);
			}
			return this;
		}

		public BuilderUpdateWrapper pageIconEq(String pageIcon) {
			if (StringUtil.isNotBlank(pageIcon)) {
				updateWrapper.eq(SaasProductPage::getPageIcon, pageIcon);
			}
			return this;
		}

		public BuilderUpdateWrapper pageUrlEq(String pageUrl) {
			if (StringUtil.isNotBlank(pageUrl)) {
				updateWrapper.eq(SaasProductPage::getPageUrl, pageUrl);
			}
			return this;
		}

		public BuilderUpdateWrapper pageAngleEq(String pageAngle) {
			if (StringUtil.isNotBlank(pageAngle)) {
				updateWrapper.eq(SaasProductPage::getPageAngle, pageAngle);
			}
			return this;
		}
		
		public BuilderUpdateWrapper pageAngleIn(String angle) {
			if (StringUtil.isNotBlank(angle)) {
				if (SaasCommonConstant.STR_STATUS_1.equals(angle)) {
					updateWrapper.in(SaasProductPage::getPageAngle, Sets.newHashSet(SaasCommonConstant.STR_STATUS_0, SaasCommonConstant.STR_STATUS_1));
				} else {
					updateWrapper.in(SaasProductPage::getPageAngle, Sets.newHashSet(SaasCommonConstant.STR_STATUS_0, SaasCommonConstant.STR_STATUS_2));
				}
			}
			return this;
		}

		public BuilderUpdateWrapper pageFloorEq(String pageFloor) {
			if (StringUtil.isNotBlank(pageFloor)) {
				updateWrapper.eq(SaasProductPage::getPageFloor, pageFloor);
			}
			return this;
		}

		public BuilderUpdateWrapper pageLevelEq(String pageLevel) {
			if (StringUtil.isNotBlank(pageLevel)) {
				updateWrapper.eq(SaasProductPage::getPageLevel, pageLevel);
			}
			return this;
		}
		
		public BuilderUpdateWrapper createTimeEq(Date createTime) {
			if (null != createTime) {
				updateWrapper.eq(SaasProductPage::getCreateTime, createTime);
			}
			return this;
		}

		public BuilderUpdateWrapper updateTimeEq(Date updateTime) {
			if (null != updateTime) {
				updateWrapper.eq(SaasProductPage::getUpdateTime, updateTime);
			}
			return this;
		}

		public BuilderUpdateWrapper updateUserEq(String updateUser) {
			if (StringUtil.isNotBlank(updateUser)) {
				updateWrapper.eq(SaasProductPage::getUpdateUser, updateUser);
			}
			return this;
		}

		public BuilderUpdateWrapper remarkEq(String remark) {
			if (StringUtil.isNotBlank(remark)) {
				updateWrapper.eq(SaasProductPage::getRemark, remark);
			}
			return this;
		}

		public LambdaUpdateWrapper<SaasProductPage> builder() {
			return updateWrapper;
		}

	}
}
