package com.persagy.account.service;

import java.util.List;

import com.persagy.common.service.ISuperService;
import com.persagy.account.pojo.dto.SaasFunction;

/**
 * 功能信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
public interface ISaasFunctionService extends ISuperService<SaasFunction> {

	/**
	 * 查询出该账号下某菜单的功能点信息集合，关联查询
	 * @param accountId
	 * @param groupCode
	 * @param menuId
	 * @return
	 */
	List<SaasFunction> queryAccountFunctionList(String accountId, String groupCode, String menuId);
	
	/**
	 * 查询出该角色下某菜单的功能点信息集合，关联查询
	 * @param groupCode
	 * @param roleId
	 * @param menuId
	 * @param pageId
	 * @return
	 */
	List<SaasFunction> queryRoleFunctionList(String groupCode, String roleId, String menuId, String pageId);

	/**
	 * 验证功能编码的唯一性
	 * 
	 * @param menuId
	 * @param pageId
	 * @param funCode
	 * @param funId
	 * @return
	 */
	boolean validFunCode(String menuId, String pageId, String funCode, String funId);
	
	/**
	 * 验证功能名称的唯一性
	 * 
	 * @param menuId
	 * @param pageId
	 * @param funName
	 * @param funId
	 * @return
	 */
	boolean validFunName(String menuId, String pageId, String funName, String funId);

	/**
	 * 获取此产品页面再此账号下所拥有的的页面权限
	 * @param accountId
	 * @param menuId
	 * @param pageId
	 * @return
	 */
	List<SaasFunction> queryProductPageFunctions(String accountId, String menuId, String pageId);

}

