package com.persagy.account.service.impl;

import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.exception.BusinessException;
import com.persagy.common.service.impl.SuperServiceImpl;
import com.persagy.account.mapper.SaasAccountProjectMapper;
import com.persagy.account.mapper.SaasAreaMapper;
import com.persagy.account.pojo.dto.SaasAccountProject;
import com.persagy.account.pojo.dto.SaasArea;
import com.persagy.account.pojo.vo.auth.AccountProjectResetVO;
import com.persagy.account.service.ISaasAccountProjectService;

/**
 * 账号-集团项目关联信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Service
public class SaasAccountProjectServiceImpl extends SuperServiceImpl<SaasAccountProjectMapper, SaasAccountProject> implements ISaasAccountProjectService {

	@Autowired
	private SaasAreaMapper saasAreaMapper;
	
	@Override
	@Transactional
	public boolean resetSaasAccountProject(String accountId, List<AccountProjectResetVO> auths) {
		LambdaQueryWrapper<SaasAccountProject> queryWrapper = new SaasAccountProject.Builder().createQueryWrapper().accountIdEq(accountId).builderQueryWrapper();
		this.baseMapper.delete(queryWrapper);
		Integer result = this.baseMapper.batchInsertSaasAccountProject(accountId, auths);
		return SqlHelper.retBool(result);
	}

	@Override
	@Transactional
	public boolean addDefaultAccountArea(String accountId, String groupCode) {
		// 先查出此集团的默认分区信息
		LambdaQueryWrapper<SaasArea> queryWrapper = new SaasArea.Builder().createQueryWrapper().groupCodeEq(groupCode).areaCodeEq(SaasCommonConstant.STR_STATUS_MINUS_1).builderQueryWrapper();
		SaasArea selectOne = this.saasAreaMapper.selectOne(queryWrapper.select(SaasArea::getId));
		if (selectOne == null) {
			throw new BusinessException("此集团不存在默认的未分区");
		}
		SaasAccountProject saasAccountProject = new SaasAccountProject();
		saasAccountProject.setAccountId(accountId);
		saasAccountProject.setGroupCode(groupCode);
		saasAccountProject.setAreaId(selectOne.getId());
		int result = this.baseMapper.insert(saasAccountProject);
		return SqlHelper.retBool(result);
	}

	@Override
	public Set<String> queryAccountAreaIds(String accountId, String groupCode) {
		return this.baseMapper.queryAccountAreaIds(accountId, groupCode);
	}

	@Override
	public boolean replaceAccountProject(List<SaasAccountProject> data) {
		Integer result = this.baseMapper.replaceAccountProject(data);
		return SqlHelper.retBool(result);
	}

}
