package com.persagy.account.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.persagy.account.mapper.SaasMenuMapper;
import com.persagy.account.mapper.SaasProductMapper;
import com.persagy.account.pojo.dto.SaasMenu;
import com.persagy.account.pojo.dto.SaasProduct;
import com.persagy.account.service.ISaasProductService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.service.impl.SuperServiceImpl;
import com.persagy.common.utils.StringUtil;

/**
 * 产品信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-04-27 16:08:25
 */
@Service
public class SaasProductServiceImpl extends SuperServiceImpl<SaasProductMapper, SaasProduct> implements ISaasProductService {

	@Autowired
	private SaasMenuMapper saasMenuMapper;
	
	@Override
	@Transactional
	public boolean updateSaasProduct(SaasProduct saasProduct, boolean isUpdateMenu) {
		Integer result = this.baseMapper.updateById(saasProduct);
		if (SqlHelper.retBool(result) && isUpdateMenu) {
			// 更新菜单图标为此产品的图标
			LambdaUpdateWrapper<SaasMenu> updateWrapper = new SaasMenu.Builder().createUpdateWrapper().productId(saasProduct.getId()).builderUpdateWrapper();
			updateWrapper.set(SaasMenu::getMenuIcon, saasProduct.getProductIcon());
			updateWrapper.set(SaasMenu::getMenuName, saasProduct.getProductName());
			this.saasMenuMapper.update(null, updateWrapper);
		}
		return SqlHelper.retBool(result);
	}
	
	@Override
	public boolean validProductCode(String appId, String productType, String productCode, String id) {
		LambdaQueryWrapper<SaasProduct> queryWrapper = new SaasProduct.BuilderQueryWrapper().appIdEq(appId).productTypeEq(productType).productCodeEq(productCode).builder();
		SaasProduct selectOne = this.baseMapper.selectOne(queryWrapper);
		if (selectOne == null) {
			return true;
		}
		if (StringUtil.isNotBlank(id) && selectOne.getId().equals(id)) {
			return true;
		}
		return false;
	}

	@Override
	public String validProductName(String appId, String productType, String productName, String id) {
		LambdaQueryWrapper<SaasProduct> queryWrapper = new SaasProduct.BuilderQueryWrapper().appIdEq(appId).productTypeEq(productType).productNameEq(productName).builder();
		SaasProduct selectOne = this.baseMapper.selectOne(queryWrapper);
		if (selectOne != null) {
			if (StringUtil.isBlank(id) || !id.equals(selectOne.getId())) {
				return "产品名称已存在";
			}
		}
		// 验证是否存在相同的菜单名称
		LambdaQueryWrapper<SaasMenu> menuWrapper = new SaasMenu.Builder().createQueryWrapper().appIdEq(appId).menuTypeEq(productType).menuNameEq(productName).builderQueryWrapper();
		SaasMenu saasMenu = this.saasMenuMapper.selectOne(menuWrapper);
		if (saasMenu != null) {
			if (StringUtil.isBlank(saasMenu.getProductId()) || !id.equals(saasMenu.getProductId())) {
				return "产品名称已在菜单中存在，不能再被关联菜单，不允许添加此名称";
			}
		}
		return null;
	}

	@Override
	@Transactional
	public boolean updateProductByMenu(String productId, String productMenu, String menuId) {
		// 未关联菜单
		LambdaUpdateWrapper<SaasProduct> updateWrapper = new SaasProduct.BuilderUpdateWrapper().idEq(productId).builder();
		updateWrapper.set(SaasProduct::getProductMenu, productMenu);
		if (SaasCommonConstant.STR_STATUS_0.equals(productMenu)) {
			updateWrapper.setSql("MENU_ID = NULL");
		} else {
			updateWrapper.set(SaasProduct::getMenuId, menuId);
		}
		Integer result = this.baseMapper.update(null, updateWrapper);
		return SqlHelper.retBool(result);
	}

}
